from typing import List, Dict

from typing_extensions import Optional

from Schemas.models.model import AnnexeModel


class PedagogicalMaterial(AnnexeModel):
    name: str
    quantities_by_grade: Dict[str, Optional[int]]


class TeacherPedagogicalMaterials(AnnexeModel):
    materials: List[PedagogicalMaterial]


class CollectiveEquipment(AnnexeModel):
    pedagogical_kits: Optional[int]
    geographic_wall_maps: Optional[int]
    metric_compendiums: Optional[int]
    geometric_compendiums: Optional[int]
    scientific_compendiums: Optional[int]
    scientific_boards: Optional[int]
    desktop_computers: Optional[int]
    image_boxes: Optional[int]
    printers: Optional[int]
    dictionaries: Optional[int]
    polyhedron_boxes: Optional[int]
    photocopiers: Optional[int]


class TeachingAndLearningEquipment(AnnexeModel):
    """"""
    collective_equipment: CollectiveEquipment
    teacher_pedagogical_materials: TeacherPedagogicalMaterials



from typing_extensions import Optional

from Schemas.models.model import AnnexeModel


class FinancialData(AnnexeModel):
    previous_year_balance: Optional[int]
    state_subsidy_amount: Optional[int]
    other_parent_contributions: Optional[int]
    other_financial_resources: Optional[int]


from typing_extensions import Optional

from Schemas.enumeration.enums import Distance, SourceEau, TypePollution
from Schemas.models.model import AnnexeModel
from Schemas.type.types import Length


class SchoolDistance(AnnexeModel):
    """Se réfère à la section Distance de l'école"""
    distance_to_department_capital: Optional[Distance]
    distance_to_commune_capital: Optional[Distance]
    distance_to_arrondissement_capital: Optional[Distance]
    distance_to_school_district_office: Optional[Distance]
    distance_to_nearest_health_center: Optional[Distance]
    distance_to_nearest_market: Optional[Distance]
    distance_to_farthest_served_village: Optional[Distance]


class SchoolAccessibility(AnnexeModel):
    permanent_access: Optional[bool]
    favorable_season_access: Optional[str]
    paved_road_distance: Optional[Length]
    all_vehicle_track_distance: Optional[Length]
    offroad_vehicle_track_distance: Optional[Length]
    two_wheeler_track_distance: Optional[Length]
    pedestrian_track_distance: Optional[Length]
    other_access_means: Optional[str]


class ServicesAndEnvironment(AnnexeModel):
    is_school_flood_prone: Optional[bool]
    is_locality_electrified: Optional[bool]
    water_supply: Optional[list[SourceEau]]
    is_school_environment_polluted: Optional[list[TypePollution]]


class NearbyEstablishments(AnnexeModel):
    distance_to_preschool: Optional[Distance]
    distance_to_first_primary_school: Optional[Distance]
    distance_to_second_primary_school: Optional[Distance]
    distance_to_secondary_school: Optional[Distance]


class SocioEnvironnement(AnnexeModel):
    """Se réfère à la section Environnement socio économique de l'école"""
    distances: SchoolDistance
    accessibility: SchoolAccessibility
    services_and_environment: ServicesAndEnvironment
    nearby_establishments: NearbyEstablishments



from fastapi import HTTPException
from pydantic_core import ValidationError
from pymongo.database import Database
from pymongo.results import UpdateResult
from starlette import status
from typing import get_type_hints

from Schemas.models.model import Model
from Schemas.models.objectid import PydanticObjectId
from Schemas.models.schools.ExamResullt import ExamResult
from Schemas.models.schools.FinancialData import FinancialData
from Schemas.models.schools.GeneralInfo import GeneralInfo
from Schemas.models.schools.IdAndLocation import IdAndLocation
from Schemas.models.schools.OfficeAndFurniture import OfficeAndFurniture
from Schemas.models.schools.ParticipatorySchoolManagement import ParticipatorySchoolManagement
from Schemas.models.schools.SocioEnvironnement import SocioEnvironnement
from Schemas.models.schools.StudentsRecord import StudentsRecord
from Schemas.models.schools.TeachingAndLearningEquipment import TeachingAndLearningEquipment
from Schemas.models.schools.TeachingStaff import TeachingStaff


class Schools(Model):
    name: str
    id_and_location: IdAndLocation
    socio_environnement: SocioEnvironnement
    general_info: GeneralInfo
    office_and_furniture: OfficeAndFurniture
    teaching_and_learning_equipment: TeachingAndLearningEquipment
    teaching_staff: TeachingStaff
    exam_results: ExamResult
    participatory_school_management: ParticipatorySchoolManagement
    financial_data: FinancialData
    students_record: StudentsRecord

    @classmethod
    def find_one(cls, database: Database, mask: dict):
        if result := database.Schools.find_one(mask):
            # Exclude _id from the result

            return Schools(**result)
        else:
            return None

    @classmethod
    def find_one_or_404(cls, database: Database, mask: dict):
        if result := database.Schools.find_one(mask):
            # Exclude _id from the result

            return Schools(**result)
        else:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="Utilisateur non trouvé",
            )

    def save(self):
        data = self.to_bson()
        result = self.database.Schools.insert_one(data)
        self.id = PydanticObjectId(result.inserted_id)

    def full_update(self, data: dict):
        self.database.Schools.update_one({"_id": self.id},
                                         {"$set": data})
        self.__init__(**Schools.find_one_or_404(self.database, {"_id": self.id}).dict())

    def partial_update(self, data: dict, section: str) -> UpdateResult:
        try:
            section_model = get_type_hints(Schools)
            validator = section_model[section]
            validator(**data)
            result = self.database.Schools.update_one({"_id": self.id},
                                                      {"$set": {section: data}})
            self.__init__(**Schools.find_one_or_404(self.database, {"_id": self.id}).dict())
            return result
        except ValidationError as e:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail=e.errors()
            )





from typing_extensions import Literal, Optional
from datetime import date
from Schemas.enumeration.enums import StudiesLevel
from Schemas.models.model import AnnexeModel
from Schemas.type.types import Year, Sexe


class HighestDegree(AnnexeModel):
    academic: Literal["CEP", "BEPC", "BAC", "LICENCE", "MASTER"]
    professional: Literal["CEAP", "CAP"]


class Seniority(AnnexeModel):
    public: int
    department: int
    school: int


class NumberOfVisit(AnnexeModel):
    inspector: int
    director: int
    cp: int


class Personnel(AnnexeModel):
    matricule: str
    name: str
    surname: str
    sexe: Sexe
    birth_year: Year
    highest_degree: HighestDegree
    grade: str
    statut: Literal["ACPDE", "ACE", "AME", "Communautaire", "Fonctionnaire de l'état", "Privé", "Autre"]
    fonction: Literal["Directeur", "Adjoint", "Enseignant"]
    real_indice: int
    teached_subjects: list[StudiesLevel]
    seniority: Seniority
    last_formation_date: date
    number_of_visit: NumberOfVisit
    up_participation: int


class TeachingStaff(AnnexeModel):
    personnel: Optional[list[Personnel]]



from pydantic import BaseModel
from typing_extensions import Optional

from Schemas.enumeration.enums import SchoolPropertyTitle
from Schemas.models.model import AnnexeModel


class PropertyAndBoundaries(BaseModel):
    """Se réfère à la sous section Propriété et délimitations de l'école"""
    school_property_title: Optional[list[SchoolPropertyTitle]]
    is_school_area_demarcated: Optional[bool]
    is_school_area_fenced: Optional[bool]


class OutdoorFacilities(BaseModel):
    """Se réfère à la sous section Aménagements extérieurs"""
    is_playground_shaded: Optional[bool]
    has_garden_or_school_field: Optional[bool]
    has_adaptations_for_special_needs_children: Optional[bool]


class WaterAndHygieneAccess(BaseModel):
    has_drinking_water_on_site: Optional[bool]
    has_handwashing_facility: Optional[bool]
    has_functional_latrines: Optional[bool]
    has_adapted_latrines_for_special_needs: Optional[bool]
    has_equipped_urinal: Optional[bool]


class SchoolServicesAndEquipment(BaseModel):
    is_electrified: Optional[bool]
    has_functional_canteen: Optional[bool]
    has_kitchen_equipment: Optional[bool]
    has_food_store_in_good_condition: Optional[bool]

    has_functional_library: Optional[bool]
    has_first_aid_kit: Optional[bool]


class ManagementBodies(AnnexeModel):
    has_parents_association: Optional[bool]
    has_mothers_association: Optional[bool]
    has_management_committee: Optional[bool]
    has_teachers_council: Optional[bool]
    has_other_management_committees: Optional[bool]


class SportsFacilities(AnnexeModel):
    has_marked_sports_field: Optional[bool]
    has_equipped_sports_field: Optional[bool]


class GeneralInfo(AnnexeModel):
    """Se réfère à la section Information générale sur l'école"""
    property_and_boundaries: PropertyAndBoundaries
    outdoor_facilities: OutdoorFacilities
    water_and_hygiene_access: WaterAndHygieneAccess
    school_services_and_equipment: SchoolServicesAndEquipment
    management_bodies: ManagementBodies
    sports_facilities: SportsFacilities



from pydantic import BaseModel
from typing_extensions import Literal, Optional

from Schemas.enumeration.enums import StudiesLevel
from Schemas.models.model import AnnexeModel

Age = Literal[
    "Moins de 5 ans",
    "5 ans",
    "6 ans",
    "7 ans",
    "8 ans",
    "9 ans",
    "10 ans",
    "11 ans",
    "12 ans",
    "13 ans",
    "14 ans",
    "15 ans",
    "16 ans et plus"
]
Redoublant = Literal[
    "Total des redoublants",
    "Redoublants ayant 6 ans",
    "Redoublants ayant 11 ans",
]
Handicape = Literal[
    "Handicapés Moteur",
    "Handicapés Sensoriel",
    "Handicapés Psychique",
    "Handicapés Mental",
    "Handicapés Maladies Invalidantes"
]
ParticularCase = Literal[
    "Étrangers",
    "Orphelins ou enfants vulnérables",
    "Orphelins",
    "Élèves sans acte de naissance",
    "Nouveaux élèves venus d'une autre école",
    "Cas de grossesse",
    "Cas de mariage précoce",
    "Cas de viol",
    "Abandon pour cause d'abus sexuel"
]


class EffectifBySex(BaseModel):
    boys: Optional[int]
    girls: Optional[int]


class StudentEnrollmentByAge(AnnexeModel):
    years: dict[Age, EffectifBySex]


class RepeatersByAge(AnnexeModel):
    status: dict[Redoublant, EffectifBySex]


class DisabledByAge(AnnexeModel):
    kind: dict[Handicape, EffectifBySex]


class ParticularSituationByAge(AnnexeModel):
    situation: dict[ParticularCase, EffectifBySex]


class StudentsRecord(AnnexeModel):
    """
    """
    student_enrollment_by_age: dict[StudiesLevel, StudentEnrollmentByAge]
    repeaters_by_age: dict[StudiesLevel, RepeatersByAge]
    disabled_by_age: dict[StudiesLevel, DisabledByAge]
    particular_situation_by_age: dict[StudiesLevel, ParticularSituationByAge]



from enum import Enum


class SchoolStatus(str, Enum):
    Public = 'Public'
    Prive_LAIC = 'Privé LAIC'
    Prive_CONFESSIONNEL = 'Privé confessionnel'
    Communautaire = 'Communautaire'


class RegionPedagogique(str, Enum):
    RP29 = "RP29"
    RP28 = "RP28"
    RP60 = "RP60"
    RP61 = "RP61"


class TypePollution(str, Enum):
    SONORE = "Sonore"
    ATMOSPHERIQUE = "Atmosphérique"


class SourceEau(str, Enum):
    EAU_COURANTE = "Eau courante"
    PUITS = "Puits"
    FORAGE = "Forage"
    CITERNE = "Citerne"


class SchoolPropertyTitle(str, Enum):
    TITRE_FONCIER = "Titre foncier"
    ACTE_DE_DONATION = "Acte de donation"
    ACTE_AUTORITE_TERRITORIALE = "Acte de l'autorité chargée de l'administration territoriale"
    CONTRAT_LOCATION = "Contrat de location entre l'école et un propriétaire"


class StudiesLevel(str, Enum):
    CI = "CI"
    CP = "CP"
    CE1 = "CE1"
    CE2 = "CE2"
    CM1 = "CM1"
    CM2 = "CM2"


class LocalType(str, Enum):
    SALLE = "Salle"
    BUREAU = "Bureau"
    MAGASIN = "Magasin"
    LATRINE = "Latrine"
    CUISINE = "Cuisine"
    BIBLIOTHEQUE = "Bibliothèque"
    LOGEMENT = "Logement"


class Distance(str, Enum):
    MOINS_DE_500_METRES = "<=0.5"
    ENTRE_500_METRES_ET_1_KM = "Entre 0.5 et 1km"
    ENTRE_1_ET_3_KM = "Entre 1 et 3km"
    PLUS_DE_3_KM = ">=3"


class SchoolKind(str, Enum):
    Classique = 'Classique'
    Inclusif = 'Inclusif'
    Specialise = 'Spécialisé'
    Autre = 'Autres'



from datetime import datetime

from pydantic import Field

from Schemas.models.model import Model


class BlacklistToken(Model):
    """
    Token Model for storing JWT tokens
    """
    token: str
    blacklisted_on: datetime = Field(default_factory=datetime.utcnow)

    def __repr__(self):
        return f'<id: token: {self.token}'

    def save(self):
        data = self.to_bson()
        result = self.database.Blacklist.insert_one(data)
        self.id = result.inserted_id

    @staticmethod
    def check_blacklist(auth_token, database):
        res = database.Blacklist.find_one({'token': auth_token})
        return bool(res)
